import user from '../../icons/user';
import clock from '../../icons/clock';
import archive from '../../icons/archive';
import pen from '../../icons/pen';

export default ( { props, data, post } ) => {
    if ( ! props || ! post ) return '';

    const { __ } = wp.i18n;
    const { 
        attributes: {
            // Thumbnail Settings
            showThumbnail,
            thumbnailSize,

            // Content Settings
            showTitle,
            showAuthor,
            showDate,
            showCategories,
            showEdit,

            // Excerpt Settings
            showExcerpt,
            excerptType,
            excerptLength,
            showReadMore,
            readMoreText,

            // Color Settings
            backgroundColor,
            backgroundColorClass,
            titleColor,
            titleColorClass,
            excerptColor,
            excerptColorClass,
            readMoreColor,
            readMoreColorClass,
            infoColor,
            infoColorClass,
            infoIconColor,
            infoIconColorClass,
            categoryColor,
            categoryColorClass,
            categoryBackgroundColor,
            categoryBackgroundColorClass,
        }
    } = props;

    // Returns the post thumbnail as img element
    function getThumbnail() {
        if ( ! post.featured_media ) return '';

        const media = wp.data.select('core').getMedia( post.featured_media );

        if ( media && media.media_details.sizes && media.media_details.sizes[ thumbnailSize ] ) {
            const size      = media.media_details.sizes[ thumbnailSize ];
            const classes   = 'attachment-' + thumbnailSize + ' size-' + thumbnailSize + ' wp-post-image';
            const img       = <img width={ size.width } height={ size.height } src={ size.source_url } className={ classes } />;

            return img;
        }
    }

    // Returns the post categories
    function getCategories( seperator ) {
        if ( ! post.categories || post.categories.length < 1 ) return '';

        let output = [];
        let counter = 0;

        post.categories.map( id => {
            let category = data.categories.find( cat => {
                return cat.id === id;
            } )
            
            if ( category && category.link ) {
                if ( ! post.featured_media ) {
                    output.push( <a href='#' 
                                    className={ infoColorClass } 
                                    style={{ color: infoColor }}>
                                    { category.name }</a> );
                } else {
                    output.push( <a href='#' 
                                className={[ categoryColorClass, categoryBackgroundColorClass ]} 
                                style={{ color: categoryColor, backgroundColor: categoryBackgroundColor }}>
                                { category.name }</a> );
                }
            }

            counter++;

            counter < post.categories.length ? output.push( seperator ) : null
        } )

        return output;
    }

    // Returns the post author
    function getAuthor() {
        if ( ! data.authors || data.authors.length < 1 ) return '';

        let author = data.authors.find( author => {
            return author.id === post.author;
        } )

        return <a href='#' className={ infoColorClass } style={{ color: infoColor }}>{ author.name }</a>;
    }

    // Returns the date in the local date format
    function getFormatedDate() {
        return new Date(post.date).toLocaleDateString();
    }

    // Classes
    const hasPostThumbnail = post.featured_media ? 'has-post-thumbnail' : '';
    let classes = [
        post.type,
        'type-' + post.type,
        'status-' + post.status,
        'format-' + post.format,
        hasPostThumbnail,
        backgroundColorClass,
    ];

    // Excerpt
    let excerpt                 = post.excerpt.rendered;
    const excerptElement        = document.createElement( 'div' );
    excerptElement.innerHTML    = excerpt;
    excerpt                     = excerptElement.textContent || excerptElement.innerText || '';

    let fullPost                = post.content.rendered;
    const fullPostElement       = document.createElement( 'div' );
    fullPostElement.innerHTML   = fullPost;
    fullPost                    = fullPostElement.textContent || fullPostElement.innerText || '';

    return(
        <article id={ 'post-' + post.id } className={ classes.join(' ') } style={{ backgroundColor: backgroundColor }}>
            { 
                showThumbnail && post.featured_media ?
                <div className="thumbnail">
                    {
                        // Categories
                        showCategories 
                        && showThumbnail
                        && post.featured_media
                        && post.categories 
                        && post.categories.length > 0 ? 
                        <div className="categories">
                            {
                                data.categories ? getCategories() : null
                            }
                        </div>
                        : null
                    }
                    <a href='#'>{ getThumbnail() }</a>
                </div>
                : null
            }
            <div className="wrapper">
                { 
                    // Title
                    showTitle ? 
                    <div className="title">
                        <a href='#' className={ titleColorClass } style={{ color: titleColor }}>
                            {post.title.rendered}
                        </a>
                    </div>
                    : null
                }
                {
                    // Excerpt
                    showExcerpt ?
                    <div className="excerpt">
                        <p className={ excerptColorClass } style={{ color: excerptColor }}>
                        { 
                            excerptType === 'excerpt' ?
                            excerptLength < excerpt.trim().split( ' ' ).length ?
                            [
                                excerpt.trim().split( ' ', excerptLength ).join( ' ' ) + ' ...\n',
                                showReadMore ? 
                                <a href='#' className={ [ 'read_more', readMoreColorClass ].join(' ') } style={{ color: readMoreColor }}>{readMoreText}</a> : 
                                null
                            ] :
                            excerpt.trim().split( ' ', excerptLength ).join( ' ' ) :
                            fullPost.trim()
                        }
                        </p> 
                    </div>
                    : null
                }
                <div className="info">
                    {
                        // Author
                        showAuthor ?
                        <div className="author">
                            <i className={ infoIconColorClass } style={{ color: infoIconColor }}>{ user }</i>
                            { getAuthor() }
                        </div>
                        : null
                    }
                    {
                        // Date
                        showDate ? 
                        <div className="date">
                            <i className={ infoIconColorClass } style={{ color: infoIconColor }}>{ clock }</i>
                            <a href='#' className={ infoColorClass } style={{ color: infoColor }}>{ getFormatedDate() }</a>
                        </div>
                        : null
                    }
                    {
                        // Categories
                        showCategories 
                        && ( ! showThumbnail || ! post.featured_media )
                        && post.categories 
                        && post.categories.length > 0 ? 
                        <div className="categories">
                            <i className={ infoIconColorClass } style={{ color: infoIconColor }}>{ archive }</i>
                            {
                                data.categories ? getCategories( ',' ) : null
                            }
                        </div>
                        : null
                    }
                    {
                        // Edit
                        showEdit ? 
                        <div className="edit">
                            <i className={ infoIconColorClass } style={{ color: infoIconColor }}>{ pen }</i>
                            <a href='#' className={ infoColorClass } style={{ color: infoColor }}>
                                { __( 'Bearbeiten', 'sv_posts' ) }
                            </a>
                        </div>
                        : null
                    }
                </div>
            </div>
        </article>
    );
}