import BlockControls from './components/block_controls';
import InspectorControls from './components/inspector_controls';
import PostList from './components/post_list';

const { withSelect } = wp.data;
const { Fragment } = wp.element;

export default withSelect( ( select, props ) => {
    const { 
        getEntityRecords,
        getPostTypes,
        getAuthors, 
    } = select( 'core' );
    const { 
        attributes: {
            postsPerPage,
            order,
            orderBy,
            excludeCurrentPost,
            filterByThumbnail,
            filterByStatus,
            filteredStatus,
            filterByType,
            filterByAuthor,
            filterByCategories,
            filteredCategories,
            filterByPosts,
            filterByDate,
            filterByDateStart,
            filterByDateEnd,
        } 
    } = props;

    // Returns an array of post ID's when this filter is set
    function getFilterByPosts() {
        if ( filterByPosts && filterByPosts.split( ',' ).length > 0 ) {
            return filterByPosts.split( ',' ).filter( ( el ) => {
                return /\S/.test( el );
            } );
        } else {
            return []; 
        }
    }

    // Returns all available post types, but filters them
    function getFilteredPostTypes() {
        const postTypes = getPostTypes();
        let filteredPostTypes = [];

        // @todo Replace filtering the result with filtering via query args, if available in getPostTypes() Method.
        for ( let key in postTypes ) {
            if ( 
                postTypes[ key ].viewable 
                && postTypes[ key ].supports.hasOwnProperty( 'editor' ) 
                && postTypes[ key ].supports.editor
            ) {
                filteredPostTypes[ key ] = postTypes[ key ];
            }
        }

        return filteredPostTypes;
    }

    // Returns posts
    function getPosts() {
        let args = {
            order: order,
            orderby: orderBy,
            per_page: postsPerPage,
            author: filterByAuthor,
            include: getFilterByPosts(),
        };

        if ( excludeCurrentPost ) {
            const post_id = wp.data.select('core/editor').getCurrentPostId();
            
            if ( post_id ) {
                args.exclude = post_id;
            }
        }

        // Filter by Categories
        if ( 
            filterByCategories 
            && filteredCategories 
            && JSON.parse( filteredCategories ).length > 0 
            && filterByType !== 'page' 
        ) {
            args.categories = JSON.parse( filteredCategories );
        }
        
        // Filter by Date
        if ( filterByDate ) {
            if ( filterByDateStart ) {
                args.after = new Date( new Date( filterByDateStart ).setUTCHours( 0, 0, 0, 0 ) ).toISOString().split('.')[0]+'Z';
            }

            if ( filterByDateEnd ) {
                args.before = new Date( new Date( filterByDateEnd ).setUTCHours( 23, 59, 59, 0 ) ).toISOString().split('.')[0]+'Z';
            }
        }

        // Filter by Status
        if ( filterByStatus ) {
            args.status = filteredStatus;
        }

        // Gets the posts
        let posts = getEntityRecords( 'postType', filterByType, args );

        // Show only posts with Thumbnail
        if ( filterByThumbnail && getEntityRecords( 'postType', filterByType, args ) ) {
            posts = posts.filter( post => {
                return post.featured_media;
            } );
        }

        return ( posts );
    }

    return {
        props,
        data: {
            posts: getPosts(),
            categories: getEntityRecords( 'taxonomy', 'category' ),
            authors: getAuthors(),
            postTypes: getFilteredPostTypes(),
        }
    }
} )( ( { props, data } ) => {
    const { __ } = wp.i18n;
    const { Placeholder } = wp.components;

    // Loading posts
    if ( ! data.posts ) {
        return (
            <Fragment>
                <InspectorControls props={ props } data={ data } />
                <Placeholder label={ __( 'Loading posts...', 'sv_posts' ) } />
            </Fragment>
        );
    }

    // No posts found
    if ( data.posts.length === 0 ) {
        return (
            <Fragment>
                <InspectorControls props={ props } data={ data } />
                <Placeholder label={ __( 'No posts found', 'sv_posts' ) } />
            </Fragment>
        );
    }

    // Found posts
    return (
        <Fragment>
            <InspectorControls props={ props } data={ data } />
            {
                typeof gutenbergPlugin !== 'undefined' ?
                <BlockControls props={ props } />
                : null
            }
            <PostList props={ props } data={ data } />
        </Fragment>
    );
} );